@@ -1,12 +1,12 @@
-# Copyright (c) 2001-2005 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
-Test cases for twisted.words.protocols.msn
+Test cases for L{twisted.words.protocols.msn}.
 """
 
 # System imports
-import StringIO, sys
+import StringIO
 
 # Twisted imports
 
@@ -24,17 +24,15 @@ else:
     from twisted.words.protocols import msn
 
 
+from twisted.python.hashlib import md5
 from twisted.protocols import loopback
 from twisted.internet.defer import Deferred
 from twisted.trial import unittest
+from twisted.test.proto_helpers import StringTransport, StringIOWithoutClosing
 
 def printError(f):
     print f
 
-class StringIOWithoutClosing(StringIO.StringIO):
-    disconnecting = 0
-    def close(self): pass
-    def loseConnection(self): pass
 
 class PassportTests(unittest.TestCase):
 
@@ -44,23 +42,30 @@ class PassportTests(unittest.TestCase):
         self.deferred.addCallback(lambda r: self.result.append(r))
         self.deferred.addErrback(printError)
 
-    def testNexus(self):
+    def test_nexus(self):
+        """
+        When L{msn.PassportNexus} receives enough information to identify the
+        address of the login server, it fires the L{Deferred} passed to its
+        initializer with that address.
+        """
         protocol = msn.PassportNexus(self.deferred, 'https://foobar.com/somepage.quux')
         headers = {
             'Content-Length' : '0',
             'Content-Type'   : 'text/html',
             'PassportURLs'   : 'DARealm=Passport.Net,DALogin=login.myserver.com/,DAReg=reg.myserver.com'
         }
-        transport = StringIOWithoutClosing()
+        transport = StringTransport()
         protocol.makeConnection(transport)
         protocol.dataReceived('HTTP/1.0 200 OK\r\n')
-        for (h,v) in headers.items(): protocol.dataReceived('%s: %s\r\n' % (h,v))
+        for (h, v) in headers.items():
+            protocol.dataReceived('%s: %s\r\n' % (h,v))
         protocol.dataReceived('\r\n')
-        self.failUnless(self.result[0] == "https://login.myserver.com/")
+        self.assertEquals(self.result[0], "https://login.myserver.com/")
+
 
     def _doLoginTest(self, response, headers):
         protocol = msn.PassportLogin(self.deferred,'foo@foo.com','testpass','https://foo.com/', 'a')
-        protocol.makeConnection(StringIOWithoutClosing())
+        protocol.makeConnection(StringTransport())
         protocol.dataReceived(response)
         for (h,v) in headers.items(): protocol.dataReceived('%s: %s\r\n' % (h,v))
         protocol.dataReceived('\r\n')
@@ -168,16 +173,15 @@ class DispatchTests(unittest.TestCase):
         client = msn.DispatchClient()
         client.userHandle = "foo"
 
-        transport = StringIOWithoutClosing()
+        transport = StringTransport()
         client.makeConnection(transport)
-        self.assertEqual(
-            transport.getvalue(), "VER 1 MSNP8 CVR0\r\n")
-        transport.seek(0)
-        transport.truncate()
+        self.assertEquals(
+            transport.value(), "VER 1 MSNP8 CVR0\r\n")
+        transport.clear()
 
         client.dataReceived(serverVersionResponse)
-        self.assertEqual(
-            transport.getvalue(),
+        self.assertEquals(
+            transport.value(),
             "CVR 2 0x0409 win 4.10 i386 MSNMSGR 5.0.0544 MSMSGS foo\r\n")
 
 
@@ -211,6 +215,7 @@ class NotificationTests(unittest.TestCase):
         self.client.factory = msn.NotificationFactory()
         self.client.state = 'START'
 
+
     def tearDown(self):
         self.client = None
 
@@ -221,16 +226,15 @@ class NotificationTests(unittest.TestCase):
         """
         self.client.factory.userHandle = "foo"
 
-        transport = StringIOWithoutClosing()
+        transport = StringTransport()
         self.client.makeConnection(transport)
-        self.assertEqual(
-            transport.getvalue(), "VER 1 MSNP8 CVR0\r\n")
-        transport.seek(0)
-        transport.truncate()
+        self.assertEquals(
+            transport.value(), "VER 1 MSNP8 CVR0\r\n")
+        transport.clear()
 
         self.client.dataReceived(serverVersionResponse)
-        self.assertEqual(
-            transport.getvalue(),
+        self.assertEquals(
+            transport.value(),
             "CVR 2 0x0409 win 4.10 i386 MSNMSGR 5.0.0544 MSMSGS foo\r\n")
 
 
@@ -255,10 +259,54 @@ class NotificationTests(unittest.TestCase):
         self._versionTest("VER 1 MSNP8\r\n")
 
 
+    def test_challenge(self):
+        """
+        L{NotificationClient} responds to a I{CHL} message by sending a I{QRY}
+        back which included a hash based on the parameters of the I{CHL}.
+        """
+        transport = StringTransport()
+        self.client.makeConnection(transport)
+        transport.clear()
+
+        challenge = "15570131571988941333"
+        self.client.dataReceived('CHL 0 ' + challenge + '\r\n')
+        # md5 of the challenge and a magic string defined by the protocol
+        response = "8f2f5a91b72102cd28355e9fc9000d6e"
+        # Sanity check - the response is what the comment above says it is.
+        self.assertEquals(
+            response, md5(challenge + "Q1P7W2E4J9R8U3S5").hexdigest())
+        self.assertEquals(
+            transport.value(),
+            # 2 is the next transaction identifier.  32 is the length of the
+            # response.
+            "QRY 2 msmsgs@msnmsgr.com 32\r\n" + response)
+
+
     def testLogin(self):
         self.client.lineReceived('USR 1 OK foo@bar.com Test%20Screen%20Name 1 0')
         self.failUnless((self.client.state == 'LOGIN'), msg='Failed to detect successful login')
 
+
+    def test_loginWithoutSSLFailure(self):
+        """
+        L{NotificationClient.loginFailure} is called if the necessary SSL APIs
+        are unavailable.
+        """
+        self.patch(msn, 'ClientContextFactory', None)
+        success = []
+        self.client.loggedIn = lambda *args: success.append(args)
+        failure = []
+        self.client.loginFailure = failure.append
+
+        self.client.lineReceived('USR 6 TWN S opaque-string-goes-here')
+        self.assertEquals(success, [])
+        self.assertEquals(
+            failure,
+            ["Exception while authenticating: "
+             "Connecting to the Passport server requires SSL, but SSL is "
+             "unavailable."])
+
+
     def testProfile(self):
         m = 'MSG Hotmail Hotmail 353\r\nMIME-Version: 1.0\r\nContent-Type: text/x-msmsgsprofile; charset=UTF-8\r\n'
         m += 'LoginTime: 1016941010\r\nEmailEnabled: 1\r\nMemberIdHigh: 40000\r\nMemberIdLow: -600000000\r\nlang_preference: 1033\r\n'
@@ -281,7 +329,7 @@ class NotificationTests(unittest.TestCase):
         # that BPRs sent as part of the SYN reply may not be interpreted
         # as such if they are for the last LST -- maybe I should
         # factor this in later.
-        self.client.makeConnection(StringIOWithoutClosing())
+        self.client.makeConnection(StringTransport())
         msn.NotificationClient.loggedIn(self.client, 'foo@foo.com', 'foobar', 1)
         lines = [
             "SYN %s 100 1 1" % self.client.currentID,
@@ -303,7 +351,7 @@ class NotificationTests(unittest.TestCase):
         c = msn.MSNContact(userHandle='userHandle@email.com')
         self.client.factory.contacts = msn.MSNContactList()
         self.client.factory.contacts.addContact(c)
-        self.client.makeConnection(StringIOWithoutClosing())
+        self.client.makeConnection(StringTransport())
         self.client.lineReceived("BPR 101 userHandle@email.com PHH 123%20456")
         c = self.client.factory.contacts.getContact('userHandle@email.com')
         self.failUnless(self.client.state == 'GOTPHONE', "Did not fire phone change callback")
@@ -316,7 +364,7 @@ class NotificationTests(unittest.TestCase):
         to be part of a SYN response (but came after the last LST)
         is received, the correct contact is updated and all is well
         """
-        self.client.makeConnection(StringIOWithoutClosing())
+        self.client.makeConnection(StringTransport())
         msn.NotificationClient.loggedIn(self.client, 'foo@foo.com', 'foo', 1)
         lines = [
             "SYN %s 100 1 1" % self.client.currentID,
@@ -374,7 +422,7 @@ class MessageHandlingTests(unittest.TestCase):
         m = msn.MSNMessage()
         m.setHeader('Content-Type', 'text/x-clientcaps')
         self.assertEquals(self.client.checkMessage(m), 0, 'Failed to detect client capability message')
-        
+
     def testTypingCheck(self):
         m = msn.MSNMessage()
         m.setHeader('Content-Type', 'text/x-msmsgscontrol')
@@ -431,37 +479,43 @@ class MessageHandlingTests(unittest.TestCase):
 
 
 class FileTransferTestCase(unittest.TestCase):
-    """ test FileSend against FileReceive """
+    """
+    test FileSend against FileReceive
+    """
 
     def setUp(self):
-        self.input = StringIOWithoutClosing()
-        self.input.writelines(['a'] * 7000)
-        self.input.seek(0)
+        self.input = 'a' * 7000
         self.output = StringIOWithoutClosing()
 
+
     def tearDown(self):
         self.input = None
         self.output = None
 
-    def testFileTransfer(self):
+
+    def test_fileTransfer(self):
+        """
+        Test L{FileSend} against L{FileReceive} using a loopback transport.
+        """
         auth = 1234
-        sender = msn.FileSend(self.input)
+        sender = msn.FileSend(StringIO.StringIO(self.input))
         sender.auth = auth
         sender.fileSize = 7000
         client = msn.FileReceive(auth, "foo@bar.com", self.output)
         client.fileSize = 7000
         def check(ignored):
-            self.failUnless((client.completed and sender.completed),
-                            msg="send failed to complete")
-            self.failUnless((self.input.getvalue() == self.output.getvalue()),
-                            msg="saved file does not match original")
+            self.assertTrue(
+                client.completed and sender.completed,
+                msg="send failed to complete")
+            self.assertEqual(
+                self.input, self.output.getvalue(),
+                msg="saved file does not match original")
         d = loopback.loopbackAsync(sender, client)
         d.addCallback(check)
         return d
 
-
 if msn is None:
-    for testClass in [PassportTests, NotificationTests,
+    for testClass in [DispatchTests, PassportTests, NotificationTests,
                       MessageHandlingTests, FileTransferTestCase]:
         testClass.skip = (
             "MSN requires an HTTP client but none is available, "
